#!/bin/bash

# checkout master

# Other Sources:
# https://raw.githubusercontent.com/Esri/file-geodatabase-api/master/FileGDB_API_1.5/FileGDB_API_1_5_VS2015.zip
# http://downloads.hexagongeospatial.com/software/2016/ECWJP2SDK/erdas-ecw-sdk-5.3.0-win.zip
# http://bin.lizardtech.com/download/developer/MrSID_DSDK-9.5.4.4703-win32-vc14.zip
# http://bin.lizardtech.com/download/developer/MrSID_DSDK-9.5.4.4703-win64-vc14.zip
# http://go.microsoft.com/fwlink/?LinkID=239647&clcid=0x409 => sqlncli-x86.msi
# http://go.microsoft.com/fwlink/?LinkID=239648&clcid=0x409 => sqlncli-x64.msi
# msiexec /i sqlncli.msi ADDLOCAL=ALL APPGUID={0CC618CE-F36A-415E-84B4-FB1BFF6967E1} (with SDK)

set -e

export PYTHON3=Python37
export MAINTAINER=JuergenFischer

umask 0002

# override msys
cat() { /bin/cat "$@"; }
cp() { /bin/cp "$@"; }
rm() { /bin/rm "$@"; }
mkdir() { /bin/mkdir -p "$@"; chmod 775 "$@"; }
tar() { /bin/tar "$@"; }
ls() { /bin/ls "$@"; }
sed() { /bin/sed "$@"; }
diff() { /bin/diff "$@"; }
find() { /bin/find "$@"; }
xargs() { /bin/xargs "$@"; }
declare -x cat cp rm mkdir tar ls sed diff

#
# run batch file and source environment changes
#
fetchenv() {
	local IFS
	IFS=
	batch=$1	
	shift
	srcenv=$(mktemp /tmp/srcenv.XXXXXXXXXX)
	dstenv=$(mktemp /tmp/dstenv.XXXXXXXXXX)
	diffenv=$(mktemp /tmp/diffenv.XXXXXXXXXX)
	cmd /c set >$srcenv
	cmd /c call `cygpath -w $batch` "$@" \>nul 2\>nul \& set >$dstenv
	diff -u $srcenv $dstenv | sed -f $D/envdiff.sed >$diffenv
	. $diffenv
	PATH=$PATH:/bin:/usr/bin:/cygdrive/c/windows32/system32:/cygdrive/c/windows
	rm -f $srcenv $dstenv $diffenv
}

export ARCH=$1
case "$ARCH" in
x86)
	export OSGEO4W_ROOT="C:\\OSGeo4W"
	BITS=32
	VCARCH=x86
	ECWARCH=win32
	FGDBARCH=
	PNG=libpng-vc14
	;;

x86_64)
	export OSGEO4W_ROOT="C:\\OSGeo4W64"
	BITS=64
	VCARCH=x64
	ECWARCH=x64
	FGDBARCH=64
	PNG=libpng
	;;

*)
	echo "Unsupported arch $ARCH" >&2
	exit 1
	;;
esac

[ -z "$ROOT" ] && ROOT=/d/src/osgeo4w64

export D=$PWD
fetchenv $OSGEO4W_ROOT/bin/o4w_env.bat

[ -z "$P" ] && P=gdal-dev
[ -z "$V" ] && V=2.4.0
[ -z "$B" ] && B=1

export P
export EXT_NMAKE_OPT=$PWD/nmake$BITS.opt
export FGDB_SDK=$ROOT/src/filegdb
export ECW_SDK=$ROOT/src/ecw-5
export MRSID_SDK=$ROOT/src/MrSID_DSDK-9.5.4.4703-win$BITS-vc14

R=$ROOT/$ARCH/release/gdal/$P
for i in oracle filegdb ecw mrsid sosi mss hdf5 pdb; do
	[ -d $R/$P-$i ] || mkdir $R/$P-$i
done
[ -d $R/python3-$P ] || mkdir $R/python3-$P

export DESTDIR=$PWD/install$BITS/apps/$P
export PY3DESTDIR=$PWD/pyinstall3-$BITS

rm -rf $PWD/install$BITS $PY3DESTDIR

fetchenv /c/Program\ Files*/Microsoft\ Visual\ Studio\ 14.0/VC/vcvarsall.bat $VCARCH
PATH=/cygdrive/c/Program\ Files*/Microsoft\ Visual\ Studio\ 14.0/Common7/IDE:$PATH

for i in bin share/gdal; do
	mkdir $DESTDIR/$i
done

mkdir install$BITS/bin
cat <<EOF >install$BITS/bin/$P-env.bat
call "%OSGEO4W_ROOT%\\bin\\proj-dev-env.bat"
SET GDAL_DATA=%OSGEO4W_ROOT%\\apps\\$P\\share\\gdal
SET GDAL_DRIVER_PATH=%OSGEO4W_ROOT%\\apps\\$P\\bin\\gdalplugins
PATH %OSGEO4W_ROOT%\\apps\\$P\\bin;%PATH%
EOF

mkdir $PY3DESTDIR/apps/$P/pymod3/scripts $PY3DESTDIR/bin
cat <<EOF >$PY3DESTDIR/bin/$P-py3-env.bat
call $P-env.bat
call py3_env.bat
SET PYTHONPATH=%OSGEO4W_ROOT%\\apps\\$P\\pymod3\\lib\\site-packages;%PYTHONPATH%
PATH %OSGEO4W_ROOT%\\apps\\$P\\pymod3\\scripts;%PATH%
EOF

cat <<EOF >$R/setup.hint
sdesc: "The GDAL/OGR library and commandline tools (nightly master)"
ldesc: "The GDAL/OGR library and commandline tools (nightly master)"
maintainer: $MAINTAINER
category: Libs Commandline_Utilities
requires: shell $PNG libjpeg proj-dev curl geos libmysql sqlite3 netcdf libpq expat xerces-c hdf4 ogdi iconv-vc14 openjpeg spatialite freexl zstd
EOF

cat <<EOF >$R/python3-$P/setup.hint
sdesc: "The GDAL/OGR Python3 Bindings and Scripts (nightly master)"
ldesc: "The GDAL/OGR Python3 Bindings and Scripts (nightly master)"
category: Libs
requires: $P python3-core python3-numpy
maintainer: $MAINTAINER
external-source: $P
EOF

cat <<EOF >$R/$P-oracle/setup.hint
sdesc: "OGR OCI and GDAL GeoRaster Plugins for Oracle (nightly master)"
ldesc: "OGR OCI and GDAL GeoRaster Plugins for Oracle (nightly master)"
category: Libs
requires: $P oci msvcrt2015 libjpeg
maintainer: $MAINTAINER
external-source: $P
EOF

cat <<EOF >$R/$P-filegdb/setup.hint
sdesc: "OGR FileGDB Driver (nightly master)"
ldesc: "OGR FileGDB Driver (nightly master)"
category: Libs
maintainer: $MAINTAINER
requires: $P msvcrt2015
external-source: $P
EOF

cat <<EOF >$R/$P-ecw/setup.hint
sdesc: "ECW Raster Plugin for GDAL (nightly master)"
ldesc: "ECW Raster Plugin for GDAL (nightly master)"
category: Libs
requires: $P
maintainer: $MAINTAINER
external-source: $P
EOF

cat <<EOF >$R/$P-mrsid/setup.hint
sdesc: "MrSID Raster Plugin for GDAL (nightly master)"
ldesc: "MrSID Raster Plugin for GDAL (nightly master)"
category: Libs
requires: $P
maintainer: $MAINTAINER 
external-source: $P
EOF

cat <<EOF >$R/$P-sosi/setup.hint
sdesc: "OGR SOSI Driver (nightly master)"
ldesc: "The OGR SOSI Driver enables OGR to read data in Norwegian SOSI standard (.sos)"
category: Libs
requires: $P
maintainer: $MAINTAINER
external-source: $P
EOF

cat <<EOF >$R/$P-mss/setup.hint
sdesc: "OGR plugin with SQL Native Client support for MSSQL Bulk Copy (nightly master)"
ldesc: "OGR plugin with SQL Native Client support for MSSQL Bulk Copy (nightly master)"
category: Libs
requires: $P msvcrt2015 msodbcsql
maintainer: $MAINTAINER
external-source: $P
EOF

cat <<EOF >$R/$P-hdf5/setup.hint
sdesc: "HDF5 plugin for GDAL (nightly master)"
ldesc: "HDF5 plugin for GDAL (nightly master)"
category: Libs
requires: $P msvcrt2015 hdf5
maintainer: $MAINTAINER
external-source: $P
EOF

cat <<EOF >$R/$P-pdb/setup.hint
sdesc: "The GDAL/OGR Debugging symbols (nightly master)"
ldesc: "The GDAL/OGR Debugging symbols (nightly master)"
category: Libs
requires: $P
maintainer: $MAINTAINER
external-source: $P
EOF

cp ../LICENSE.TXT $R/$P-$V-$B.txt
cp ../LICENSE.TXT $R/$P-oracle/$P-oracle-$V-$B.txt
cp ../LICENSE.TXT $R/$P-mss/$P-mss-$V-$B.txt
cp ../LICENSE.TXT $R/$P-pdb/$P-pdb-$V-$B.txt
cp ../LICENSE.TXT $R/python3-$P/python3-$P-$V-$B.txt
cp $FGDB_SDK/license/userestrictions.txt $R/$P-filegdb/$P-filegdb-$V-$B.txt
cp $ECW_SDK/EULA.rtf $R/$P-ecw/$P-ecw-$V-$B.rtf
cp $MRSID_SDK/LICENSE.pdf $R/$P-mrsid/$P-mrsid-$V-$B.pdf

cd ..

export v=${V%.*}
export v=$(printf "%d%02d" ${v%.*} ${v#*.})

for i in clean default install devinstall; do
	[ -f ../osgeo4w/no$i ] && continue

	nmake /f makefile.vc \
		CC=$(cygpath -w $OSGEO4W_ROOT/bin/clcache.bat) \
		CPP=$(cygpath -w $OSGEO4W_ROOT/bin/clcache.bat) \
		VERSION=$v \
		EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) \
		GDAL_HOME=$(cygpath -w $DESTDIR) \
		FGDB_SDK=$(cygpath -w $FGDB_SDK) \
		ECWDIR=$(cygpath -w $ECW_SDK) \
		MRSID_SDK=$(cygpath -w $MRSID_SDK) \
		$i
done

cd swig

(
	fetchenv $OSGEO4W_ROOT/bin/py3_env.bat

	if [ "$ARCH" = "x86" ]; then
		export LIB="c:/Program Files (x86)/Microsoft Visual Studio 14.0/VC/lib"
	else
		export LIB="c:/Program Files (x86)/Microsoft Visual Studio 14.0/VC/lib/amd64"
	fi
	export INCLUDE="c:/Program Files (x86)/Microsoft Visual Studio 14.0/VC/include"

	echo LIB:$LIB
	echo INCLUDE:$INCLUDE
	nmake /f makefile.vc SWIG=$(cygpath -w $OSGEO4W_ROOT/bin/swig.bat) PYDIR=$(cygpath -w $OSGEO4W_ROOT/apps/$PYTHON3) EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) clean python

	cd python

	# Fix CRLF => CRLFCR conversion.
	find build -name "*.py" -print | xargs /bin/flip -u

	mkdir $PY3DESTDIR/apps/$P/pymod3
	python setup.py install --prefix=$(cygpath -w $PY3DESTDIR/apps/$P/pymod3) --single-version-externally-managed --record ..\\..\\osgeo4w\\log\\record3-$ARCH.log
)

cd ../osgeo4w

cp $FGDB_SDK/bin$FGDBARCH/FileGDBAPI.dll $DESTDIR/bin
cp $ECW_SDK/bin/vc140/$ECWARCH/NCSEcw.dll $DESTDIR/bin
cp $MRSID_SDK/Raster_DSDK/lib/lti_dsdk_cdll_9.5.dll $DESTDIR/bin
cp $MRSID_SDK/Raster_DSDK/lib/tbb.dll $DESTDIR/bin
cp $MRSID_SDK/Raster_DSDK/lib/lti_dsdk_9.5.dll $DESTDIR/bin
#cp $MRSID_SDK/Lidar_DSDK/lib/lti_lidar_dsdk_1.1.dll $DESTDIR/bin

tar -C $PY3DESTDIR --exclude="*.pyc" --exclude __pycache__ -cjvf $R/python3-$P/python3-$P-$V-$B.tar.bz2 \
	apps/$P/pymod3 \
	bin/$P-py3-env.bat
tar -C install$BITS --remove-files -cjvf $R/$P-filegdb/$P-filegdb-$V-$B.tar.bz2 \
	apps/$P/bin/gdalplugins/ogr_FileGDB.dll \
	apps/$P/bin/FileGDBAPI.dll
tar -C install$BITS --remove-files -cjvf $R/$P-sosi/$P-sosi-$V-$B.tar.bz2 \
	apps/$P/bin/gdalplugins/ogr_SOSI.dll
tar -C install$BITS --remove-files -cjvf $R/$P-oracle/$P-oracle-$V-$B.tar.bz2 \
	apps/$P/bin/gdalplugins/gdal_GEOR.dll \
	apps/$P/bin/gdalplugins/ogr_OCI.dll
tar -C install$BITS --remove-files -cjvf $R/$P-mss/$P-mss-$V-$B.tar.bz2 \
	apps/$P/bin/gdalplugins/ogr_MSSQLSpatial.dll
tar -C install$BITS --remove-files -cjvf $R/$P-ecw/$P-ecw-$V-$B.tar.bz2 \
	apps/$P/bin/gdalplugins/gdal_ECW_JP2ECW.dll \
	apps/$P/bin/NCSEcw.dll
tar -C install$BITS --remove-files -cjvf $R/$P-mrsid/$P-mrsid-$V-$B.tar.bz2 \
	apps/$P/bin/gdalplugins/gdal_MrSID.dll \
	apps/$P/bin/lti_dsdk_cdll_9.5.dll \
	apps/$P/bin/lti_dsdk_9.5.dll \
	apps/$P/bin/tbb.dll

#	apps/$P/bin/gdalplugins/gdal_MG4Lidar.dll \
#	apps/$P/bin/lti_lidar_dsdk_1.1.dll \

tar -C install$BITS --remove-files -cjvf $R/$P-hdf5/$P-hdf5-$V-$B.tar.bz2 \
	apps/$P/bin/gdalplugins/gdal_HDF5.dll
tar -C install$BITS --remove-files -cjvf $R/$P-pdb/$P-pdb-$V-$B.tar.bz2 \
	apps/$P/lib/gdal$v.pdb
tar -C install$BITS --remove-files -cjvf $R/$P-$V-$B.tar.bz2 \
	bin \
	apps/$P/bin \
	apps/$P/include \
	apps/$P/lib \
	apps/$P/share
tar -C .. -cjvf $R/$P-$V-$B-src.tar.bz2 \
	osgeo4w/nmake$BITS.opt \
	osgeo4w/package.sh \
	osgeo4w/envdiff.sed
